@@ -45,12 +45,21 @@ try:
 except:
     pass
 
+def byte_regexp(bitmask, bitvalues):
+    r = b'['
+    for value in range(256):
+        if value & bitmask == bitvalues:
+            # Generates unencoded string (bytes) in both Python 2 and 3.
+            r += bytes(bytearray([value]))
+    r += b']'
+    return r
+
 class Endianess(Enum):
     _enum_ = 'LITTLE BIG'
 
 class Architecture(AbstractSingleton):
 
-    def __init__(self, arch, mode, addressLength, align, endianess=Endianess.LITTLE):
+    def __init__(self, arch, mode, addressLength, align, endianess=Endianess.LITTLE, branch_delay_slot=False):
         super(Architecture, self).__init__()
         self._name = 'raw'
         self._arch = arch
@@ -76,6 +85,7 @@ class Architecture(AbstractSingleton):
         self._initCategories()
 
         self._initEndianess(endianess)
+        self._hasBranchDelaySlot = branch_delay_slot
 
         self._endings[gadget.GadgetType.ALL] = self._endings[
             gadget.GadgetType.ROP] + self._endings[gadget.GadgetType.JOP] + self._endings[gadget.GadgetType.SYS]
@@ -144,6 +154,10 @@ class Architecture(AbstractSingleton):
     def endianess(self):
         return self._endianess
 
+    @property
+    def hasBranchDelaySlot(self):
+        return self._hasBranchDelaySlot
+
     def getRegisterName(self, reg):
         if self.info is None:
             return reg
@@ -197,22 +211,32 @@ class ArchitectureX86(Architecture):
                                                 (b'\x65\xff\x15\x10\x00\x00\x00', 7)]       # call gs:[10]
 
         self._endings[gadget.GadgetType.JOP] = [(
-            b'\xff[\x20\x21\x22\x23\x26\x27]', 2),
-            (b'\xff[\xe0\xe1\xe2\xe3\xe4\xe6\xe7]', 2),
-            (b'\xff[\x10\x11\x12\x13\x16\x17]', 2),
-            (b'\xff[\xd0\xd1\xd2\xd3\xd4\xd6\xd7]', 2),
-            (b'\xff[\x14\x24]\x24', 3),
-            (b'\xff[\x55\x65]\x00', 3),
-            (b'\xff[\xa0\xa1\xa2\xa3\xa6\xa7][\x00-\x0ff]{4}', 6),
+            b'\xff[\x20\x21\x22\x23\x26\x27]', 2),                                      # jmp [reg]
+            (b'\xf2\xff[\x20\x21\x22\x23\x26\x27]', 3),                                      # bnd jmp [reg]
+            (b'\xff[\xe0\xe1\xe2\xe3\xe4\xe6\xe7]', 2),                                 # jmp reg
+            (b'\xf2\xff[\xe0\xe1\xe2\xe3\xe4\xe6\xe7]', 3),                                 # bnd jmp reg
+            (b'\xff[\x10\x11\x12\x13\x16\x17]', 2),                                     # call [reg]
+            (b'\xf2\xff[\x10\x11\x12\x13\x16\x17]', 3),                                     # bnd call [reg]
+            (b'\xff[\xd0\xd1\xd2\xd3\xd4\xd6\xd7]', 2),                                 # call reg
+            (b'\xf2\xff[\xd0\xd1\xd2\xd3\xd4\xd6\xd7]', 3),                                 # bnd call reg
+            (b'\xff[\x14\x24]\x24', 3),                                                 # call/jmp [esp]
+            (b'\xf2\xff[\x14\x24]\x24', 4),                                                 # bnd call/jmp [esp]
+            (b'\xff[\x55\x65]\x00', 3),                                                 # call/jmp [ebp]
+            (b'\xf2\xff[\x55\x65]\x00', 4),                                             # bnd call/jmp [ebp]
+            (b'\xff[\xa0\xa1\xa2\xa3\xa6\xa7][\x00-\x0ff]{4}', 6),                      # jmp [reg+value]
+            (b'\xf2\xff[\xa0\xa1\xa2\xa3\xa6\xa7][\x00-\x0ff]{4}', 7),                  # bnd jmp [reg+value]
             (b'\xff\xa4\x24[\x00-\xff]{4}', 7),
+            (b'\xf2\xff\xa4\x24[\x00-\xff]{4}', 8),
             (b'\xff[\x50-\x53\x55-\x57][\x00-\xff]{1}', 3),                             # call [reg + value]
+            (b'\xf2\xff[\x50-\x53\x55-\x57][\x00-\xff]{1}', 4),                             # call [reg + value]
             (b'\xff[\x60-\x63\x65-\x67][\x00-\xff]{1}', 3),                             # jmp [reg + value]
+            (b'\xf2\xff[\x60-\x63\x65-\x67][\x00-\xff]{1}', 4),                             # jmp [reg + value]
             #(b'\xe9[\x00-\xff]{4}', 5),                                                 # jmp value
             #(b'\xe8[\x00-\xff]{4}', 5),                                                 # call value
             (b'\xff[\x90\x91\x92\x93\x94\x96\x97][\x00-\x0ff]{4}', 6)]
 
     def _initBadInstructions(self):
-        self._badInstructions = ['retf','enter','loop','loopne','int3', 'db', 'ret', 'jmp', 'les', 'lds', 'jle','jl', 'jb','jbe','jg','jge','ja','jae', 'jne', 'je', 'js']
+        self._badInstructions = ['enter','loop','loopne','int3', 'db', 'ret', 'jmp']
 
     def _initCategories(self):
         self._categories = {
@@ -289,7 +313,7 @@ class ArchitectureX86_64(ArchitectureX86):
 class ArchitectureMips(Architecture):
 
     def __init__(self, endianess=Endianess.LITTLE):
-        super(ArchitectureMips,self).__init__(CS_ARCH_MIPS, CS_MODE_32, 4, 4, endianess)
+        super(ArchitectureMips,self).__init__(CS_ARCH_MIPS, CS_MODE_32, 4, 4, endianess, branch_delay_slot=True)
         self._name = 'MIPS'
 
         if 'keystone' in globals():
@@ -434,7 +458,7 @@ class ArchitectureArm64(Architecture):
 class ArchitecturePPC(Architecture):
 
     def __init__(self):
-        super(ArchitecturePPC, self).__init__(CS_ARCH_PPC , CS_MODE_32 + CS_MODE_BIG_ENDIAN, 4, 4)
+        super(ArchitecturePPC, self).__init__(CS_ARCH_PPC, CS_MODE_32 + CS_MODE_BIG_ENDIAN, 4, 4)
         self._name = 'PPC'
 
         if 'keystone' in globals():
@@ -443,7 +467,8 @@ class ArchitecturePPC(Architecture):
     def _initGadgets(self):
         super(ArchitecturePPC, self)._initGadgets()
         self._endings[gadget.GadgetType.ROP] = [(b'\x4e\x80\x00\x20', 4)] # blr
-        self._endings[gadget.GadgetType.JOP] = []
+        self._endings[gadget.GadgetType.JOP] = [(b'\x4e\x80\x04[\x20-\x21]', 4)] # bctr, bctrl
+        self._endings[gadget.GadgetType.SYS] = [(b'\x44\x00\x00\x02', 4)] # sc
 
 
 class ArchitecturePPC64(ArchitecturePPC):
@@ -456,6 +481,42 @@ class ArchitecturePPC64(ArchitecturePPC):
         if 'keystone' in globals():
             self._ksarch = (keystone.KS_ARCH_PPC, keystone.KS_MODE_64)
 
+class ArchitectureSPARC(Architecture):
+
+    def __init__(self):
+        super(ArchitectureSPARC, self).__init__(CS_ARCH_SPARC, CS_MODE_32 + CS_MODE_BIG_ENDIAN, 4, 4,
+						branch_delay_slot=True)
+        self._name = 'SPARC'
+
+        if 'keystone' in globals():
+            self._ksarch = (keystone.KS_ARCH_SPARC, keystone.KS_MODE_32)
+
+    def _initGadgets(self):
+        super(ArchitectureSPARC, self)._initGadgets()
+        self._endings[gadget.GadgetType.ROP] = []
+        self._endings[gadget.GadgetType.JOP] = []
+        self._endings[gadget.GadgetType.SYS] = []
+
+
+class ArchitectureSPARC64(ArchitectureSPARC):
+
+    def __init__(self):
+
+        Architecture.__init__(self, CS_ARCH_SPARC, CS_MODE_V9 + CS_MODE_BIG_ENDIAN, 4, 4, branch_delay_slot=True)
+        self._name = 'SPARC64'
+
+        if 'keystone' in globals():
+            self._ksarch = (keystone.KS_ARCH_SPARC, keystone.KS_MODE_64)
+
+    def _initGadgets(self):
+        super(ArchitectureSPARC, self)._initGadgets()
+        self._endings[gadget.GadgetType.ROP] = [
+            (byte_regexp(0b11000001, 0b10000001) + b'[\xc8-\xcf][\x00-\xff][\x00-\xff]', 4), # return
+            (byte_regexp(0b11000001, 0b10000001) + b'[\xe8-\xef][\x00-\xff][\x00-\xff]', 4)] # restore
+        self._endings[gadget.GadgetType.JOP] = [
+            (byte_regexp(0b11000001, 0b10000001) + b'[\xc0-\xc7][\x00-\xff][\x00-\xff]', 4)] # jmpl (ret, retl)
+        self._endings[gadget.GadgetType.SYS] = [(b'\x91\xd0\x20\x6d', 4)] # ta 0x6d
+
 
 
 x86 = ArchitectureX86()
@@ -470,6 +531,7 @@ ARMTHUMB = ArchitectureArmThumb()
 ARM64 = ArchitectureArm64()
 PPC = ArchitecturePPC()
 PPC64 = ArchitecturePPC64()
+SPARC64 = ArchitectureSPARC64()
 
 def getArchitecture(archString):
     arch = globals().get(archString, None)
@@ -477,4 +539,4 @@ def getArchitecture(archString):
     if isinstance(arch, Architecture):
         return arch
 
-    raise NotSupportedError('Architecture is not supported: ' + archString + '\nSupported architectures are: x86, x86_64, MIPS, MIPS64, ARM, ARMTHUMB, ARM64, PPC, PPC64')
+    raise NotSupportedError('Architecture is not supported: ' + archString + '\nSupported architectures are: x86, x86_64, MIPS, MIPS64, ARM, ARMTHUMB, ARM64, PPC, PPC64, SPARC64')
